#line 2 "sqlscanner.cpp"

#line 4 "sqlscanner.cpp"

#define  YY_INT_ALIGNED short int

/* A lexical scanner generated by flex */

#define FLEX_SCANNER
#define YY_FLEX_MAJOR_VERSION 2
#define YY_FLEX_MINOR_VERSION 5
#define YY_FLEX_SUBMINOR_VERSION 33
#if YY_FLEX_SUBMINOR_VERSION > 0
#define FLEX_BETA
#endif

/* First, we deal with  platform-specific or compiler-specific issues. */

/* begin standard C headers. */
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>

/* end standard C headers. */

/* flex integer type definitions */

#ifndef FLEXINT_H
#define FLEXINT_H

/* C99 systems have <inttypes.h>. Non-C99 systems may or may not. */

#if __STDC_VERSION__ >= 199901L

/* C99 says to define __STDC_LIMIT_MACROS before including stdint.h,
 * if you want the limit (max/min) macros for int types.
 */
#ifndef __STDC_LIMIT_MACROS
#define __STDC_LIMIT_MACROS 1
#endif

#include <inttypes.h>
typedef int8_t flex_int8_t;
typedef uint8_t flex_uint8_t;
typedef int16_t flex_int16_t;
typedef uint16_t flex_uint16_t;
typedef int32_t flex_int32_t;
typedef uint32_t flex_uint32_t;
#else
typedef signed char flex_int8_t;
typedef short int flex_int16_t;
typedef int flex_int32_t;
typedef unsigned char flex_uint8_t;
typedef unsigned short int flex_uint16_t;
typedef unsigned int flex_uint32_t;
#endif /* ! C99 */

/* Limits of integral types. */
#ifndef INT8_MIN
#define INT8_MIN               (-128)
#endif
#ifndef INT16_MIN
#define INT16_MIN              (-32767-1)
#endif
#ifndef INT32_MIN
#define INT32_MIN              (-2147483647-1)
#endif
#ifndef INT8_MAX
#define INT8_MAX               (127)
#endif
#ifndef INT16_MAX
#define INT16_MAX              (32767)
#endif
#ifndef INT32_MAX
#define INT32_MAX              (2147483647)
#endif
#ifndef UINT8_MAX
#define UINT8_MAX              (255U)
#endif
#ifndef UINT16_MAX
#define UINT16_MAX             (65535U)
#endif
#ifndef UINT32_MAX
#define UINT32_MAX             (4294967295U)
#endif

#endif /* ! FLEXINT_H */

#ifdef __cplusplus

/* The "const" storage-class-modifier is valid. */
#define YY_USE_CONST

#else /* ! __cplusplus */

#if __STDC__

#define YY_USE_CONST

#endif /* __STDC__ */
#endif /* ! __cplusplus */

#ifdef YY_USE_CONST
#define yyconst const
#else
#define yyconst
#endif

/* Returned upon end-of-file. */
#define YY_NULL 0

/* Promotes a possibly negative, possibly signed char to an unsigned
 * integer for use as an array index.  If the signed char is negative,
 * we want to instead treat it as an 8-bit unsigned char, hence the
 * double cast.
 */
#define YY_SC_TO_UI(c) ((unsigned int) (unsigned char) c)

/* Enter a start condition.  This macro really ought to take a parameter,
 * but we do it the disgusting crufty way forced on us by the ()-less
 * definition of BEGIN.
 */
#define BEGIN (yy_start) = 1 + 2 *

/* Translate the current start state into a value that can be later handed
 * to BEGIN to return to the state.  The YYSTATE alias is for lex
 * compatibility.
 */
#define YY_START (((yy_start) - 1) / 2)
#define YYSTATE YY_START

/* Action number for EOF rule of a given start state. */
#define YY_STATE_EOF(state) (YY_END_OF_BUFFER + state + 1)

/* Special action meaning "start processing a new file". */
#define YY_NEW_FILE yyrestart(yyin  )

#define YY_END_OF_BUFFER_CHAR 0

/* Size of default input buffer. */
#ifndef YY_BUF_SIZE
#define YY_BUF_SIZE 16384
#endif

/* The state buf must be large enough to hold one state per character in the main buffer.
 */
#define YY_STATE_BUF_SIZE   ((YY_BUF_SIZE + 2) * sizeof(yy_state_type))

#ifndef YY_TYPEDEF_YY_BUFFER_STATE
#define YY_TYPEDEF_YY_BUFFER_STATE
typedef struct yy_buffer_state *YY_BUFFER_STATE;
#endif

extern int yyleng;

extern FILE *yyin, *yyout;

#define EOB_ACT_CONTINUE_SCAN 0
#define EOB_ACT_END_OF_FILE 1
#define EOB_ACT_LAST_MATCH 2

#define YY_LESS_LINENO(n)

/* Return all but the first "n" matched characters back to the input stream. */
#define yyless(n) \
    do \
    { \
        /* Undo effects of setting up yytext. */ \
        int yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
        *yy_cp = (yy_hold_char); \
        YY_RESTORE_YY_MORE_OFFSET \
        (yy_c_buf_p) = yy_cp = yy_bp + yyless_macro_arg - YY_MORE_ADJ; \
        YY_DO_BEFORE_ACTION; /* set up yytext again */ \
    } \
    while ( 0 )

#define unput(c) yyunput( c, (yytext_ptr)  )

/* The following is because we cannot portably get our hands on size_t
 * (without autoconf's help, which isn't available because we want
 * flex-generated scanners to compile on their own).
 */

#ifndef YY_TYPEDEF_YY_SIZE_T
#define YY_TYPEDEF_YY_SIZE_T
typedef unsigned int yy_size_t;
#endif

#ifndef YY_STRUCT_YY_BUFFER_STATE
#define YY_STRUCT_YY_BUFFER_STATE
struct yy_buffer_state {
    FILE *yy_input_file;

    char *yy_ch_buf;  /* input buffer */
    char *yy_buf_pos;  /* current position in input buffer */

    /* Size of input buffer in bytes, not including room for EOB
     * characters.
     */
    yy_size_t yy_buf_size;

    /* Number of characters read into yy_ch_buf, not including EOB
     * characters.
     */
    int yy_n_chars;

    /* Whether we "own" the buffer - i.e., we know we created it,
     * and can realloc() it to grow it, and should free() it to
     * delete it.
     */
    int yy_is_our_buffer;

    /* Whether this is an "interactive" input source; if so, and
     * if we're using stdio for input, then we want to use getc()
     * instead of fread(), to make sure we stop fetching input after
     * each newline.
     */
    int yy_is_interactive;

    /* Whether we're considered to be at the beginning of a line.
     * If so, '^' rules will be active on the next match, otherwise
     * not.
     */
    int yy_at_bol;

    int yy_bs_lineno; /**< The line count. */
    int yy_bs_column; /**< The column count. */

    /* Whether to try to fill the input buffer when we reach the
     * end of it.
     */
    int yy_fill_buffer;

    int yy_buffer_status;

#define YY_BUFFER_NEW 0
#define YY_BUFFER_NORMAL 1
    /* When an EOF's been seen but there's still some text to process
     * then we mark the buffer as YY_EOF_PENDING, to indicate that we
     * shouldn't try reading from the input source any more.  We might
     * still have a bunch of tokens to match, though, because of
     * possible backing-up.
     *
     * When we actually see the EOF, we change the status to "new"
     * (via yyrestart()), so that the user can continue scanning by
     * just pointing yyin at a new input file.
     */
#define YY_BUFFER_EOF_PENDING 2

};
#endif /* !YY_STRUCT_YY_BUFFER_STATE */

/* Stack of input buffers. */
static size_t yy_buffer_stack_top = 0; /**< index of top of stack. */
static size_t yy_buffer_stack_max = 0; /**< capacity of stack. */
static YY_BUFFER_STATE * yy_buffer_stack = 0; /**< Stack as an array. */

/* We provide macros for accessing buffer states in case in the
 * future we want to put the buffer states in a more general
 * "scanner state".
 *
 * Returns the top of the stack, or NULL.
 */
#define YY_CURRENT_BUFFER ( (yy_buffer_stack) \
                            ? (yy_buffer_stack)[(yy_buffer_stack_top)] \
                            : NULL)

/* Same as previous macro, but useful when we know that the buffer stack is not
 * NULL or when we need an lvalue. For internal use only.
 */
#define YY_CURRENT_BUFFER_LVALUE (yy_buffer_stack)[(yy_buffer_stack_top)]

/* yy_hold_char holds the character lost when yytext is formed. */
static char yy_hold_char;
static int yy_n_chars;  /* number of characters read into yy_ch_buf */
int yyleng;

/* Points to current character in buffer. */
static char *yy_c_buf_p = (char *) 0;
static int yy_init = 0;  /* whether we need to initialize */
static int yy_start = 0; /* start state number */

/* Flag which is used to allow yywrap()'s to do buffer switches
 * instead of setting up a fresh yyin.  A bit of a hack ...
 */
static int yy_did_buffer_switch_on_eof;

void yyrestart(FILE *input_file);
void yy_switch_to_buffer(YY_BUFFER_STATE new_buffer);
YY_BUFFER_STATE yy_create_buffer(FILE *file, int size);
void yy_delete_buffer(YY_BUFFER_STATE b);
void yy_flush_buffer(YY_BUFFER_STATE b);
void yypush_buffer_state(YY_BUFFER_STATE new_buffer);
void yypop_buffer_state(void);

static void yyensure_buffer_stack(void);
static void yy_load_buffer_state(void);
static void yy_init_buffer(YY_BUFFER_STATE b, FILE *file);

#define YY_FLUSH_BUFFER yy_flush_buffer(YY_CURRENT_BUFFER )

YY_BUFFER_STATE yy_scan_buffer(char *base, yy_size_t size);
YY_BUFFER_STATE yy_scan_string(yyconst char *yy_str);
YY_BUFFER_STATE yy_scan_bytes(yyconst char *bytes, int len);

void *yyalloc(yy_size_t);
void *yyrealloc(void *, yy_size_t);
void yyfree(void *);

#define yy_new_buffer yy_create_buffer

#define yy_set_interactive(is_interactive) \
    { \
        if ( ! YY_CURRENT_BUFFER ){ \
            yyensure_buffer_stack (); \
            YY_CURRENT_BUFFER_LVALUE =    \
                                          yy_create_buffer(yyin,YY_BUF_SIZE ); \
        } \
        YY_CURRENT_BUFFER_LVALUE->yy_is_interactive = is_interactive; \
    }

#define yy_set_bol(at_bol) \
    { \
        if ( ! YY_CURRENT_BUFFER ){\
            yyensure_buffer_stack (); \
            YY_CURRENT_BUFFER_LVALUE =    \
                                          yy_create_buffer(yyin,YY_BUF_SIZE ); \
        } \
        YY_CURRENT_BUFFER_LVALUE->yy_at_bol = at_bol; \
    }

#define YY_AT_BOL() (YY_CURRENT_BUFFER_LVALUE->yy_at_bol)

/* Begin user sect3 */

#define yywrap(n) 1
#define YY_SKIP_YYWRAP

typedef unsigned char YY_CHAR;

FILE *yyin = (FILE *) 0, *yyout = (FILE *) 0;

typedef int yy_state_type;

extern int yylineno;

int yylineno = 1;

extern char *yytext;
#define yytext_ptr yytext

static yy_state_type yy_get_previous_state(void);
static yy_state_type yy_try_NUL_trans(yy_state_type current_state);
static int yy_get_next_buffer(void);
static void yy_fatal_error(yyconst char msg[]);

/* Done after the current pattern has been matched and before the
 * corresponding action - sets up yytext.
 */
#define YY_DO_BEFORE_ACTION \
    (yytext_ptr) = yy_bp; \
    yyleng = (size_t) (yy_cp - yy_bp); \
    (yy_hold_char) = *yy_cp; \
    *yy_cp = '\0'; \
    (yy_c_buf_p) = yy_cp;

#define YY_NUM_RULES 43
#define YY_END_OF_BUFFER 44
/* This struct is not used in this scanner,
   but its presence is necessary. */
struct yy_trans_info {
    flex_int32_t yy_verify;
    flex_int32_t yy_nxt;
};
static yyconst flex_int16_t yy_accept[148] = {   0,
        0,    0,   44,   43,   41,   42,   43,   42,   42,   43,
        42,    7,   42,   42,   42,   42,   39,   39,   39,   39,
        39,   39,   39,   39,   39,   39,   39,   39,   39,   39,
        39,   39,   42,   41,    2,    0,   38,    9,    0,    8,
        8,    7,   39,   27,    4,    1,    3,    5,   28,    0,
        39,   10,   35,   39,   39,   39,    6,   22,   39,   39,
        39,   39,   39,   24,   25,   39,   39,   39,   39,   39,
        39,   26,    8,   40,    9,   36,   39,   39,   39,   39,
        0,   39,   39,   39,   21,   39,   39,   39,   39,   39,
        39,   39,   29,   39,   37,   12,   39,    0,   14,   15,

        16,    0,   23,   39,   39,   39,   39,   39,   39,   39,
        39,    0,    0,    0,   34,   30,   39,   39,   32,   33,
        11,   39,    0,    0,    0,   31,   39,   13,    0,   20,
        0,   39,    0,    0,    0,    0,    0,    0,    0,    0,
        18,   19,    0,    0,    0,   17,    0
                                             } ;

static yyconst flex_int32_t yy_ec[256] = {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    2,    2,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    2,    3,    4,    5,    1,    5,    6,    7,    5,
        5,    5,    5,    5,    5,    8,    5,    9,    9,    9,
        9,    9,    9,    9,    9,    9,    9,    5,    5,   10,
        11,   12,    5,    5,   16,   17,   18,   19,   20,   21,
        22,   23,   24,   25,   26,   27,   28,   29,   30,   15,
        15,   31,   32,   33,   34,   15,   35,   36,   37,   15,
        13,    1,   14,    5,   15,    5,   16,   17,   18,   19,

        20,   21,   22,   23,   24,   25,   26,   27,   28,   29,
        30,   15,   15,   31,   32,   33,   34,   15,   35,   36,
        37,   15,    1,   38,    1,    5,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,

        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1
                                         } ;

static yyconst flex_int32_t yy_meta[39] = {   0,
        1,    1,    1,    1,    1,    1,    1,    2,    2,    1,
        1,    1,    3,    3,    2,    2,    2,    2,    2,    2,
        2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
        2,    2,    2,    2,    2,    2,    2,    1
                                          } ;

static yyconst flex_int16_t yy_base[152] = {   0,
        0,    0,  254,  255,  251,  241,  247,  255,  244,  242,
        239,   31,   31,  236,   33,    0,  238,   38,   39,   40,
        41,   42,   43,   44,   45,   47,   49,   55,   74,   46,
        60,   56,  207,  242,  255,  239,  255,  255,  235,  232,
        231,   50,  231,  255,  255,  255,  255,  255,  255,  224,
        76,   48,  229,   77,   79,   80,   81,   88,   83,   84,
        91,   85,   92,  228,   93,   94,   95,   96,   98,  100,
        101,  255,  226,  255,  226,  225,  105,  115,  117,  118,
        125,  120,  122,  121,  129,  126,  127,  128,  132,  134,
        135,  136,  224,  131,  223,  222,  138,  127,  221,  220,

        219,  146,  218,  142,  148,  151,  155,  157,  160,  163,
        164,  192,  188,  190,  205,  204,  158,  171,  203,  202,
        201,  162,  206,  179,  177,  196,  166,  195,  173,  255,
        177,  184,  166,  172,  174,  171,  180,  165,  167,  159,
        255,  255,  187,  183,  158,  255,  255,  216,  219,   58,
        222
                                           } ;

static yyconst flex_int16_t yy_def[152] = {   0,
        147,    1,  147,  147,  147,  147,  148,  147,  147,  149,
        147,  150,  147,  147,  147,  151,  150,  150,  150,  150,
        150,  150,  150,  150,  150,  150,  150,  150,  150,  150,
        150,  150,  147,  147,  147,  148,  147,  147,  149,  147,
        147,  150,  150,  147,  147,  147,  147,  147,  147,  151,
        150,  150,  150,  150,  150,  150,  150,  150,  150,  150,
        150,  150,  150,  150,  150,  150,  150,  150,  150,  150,
        150,  147,  147,  147,  150,  150,  150,  150,  150,  150,
        147,  150,  150,  150,  150,  150,  150,  150,  150,  150,
        150,  150,  150,  150,  150,  150,  150,  147,  150,  150,

        150,  147,  150,  150,  150,  150,  150,  150,  150,  150,
        150,  147,  147,  147,  150,  150,  150,  150,  150,  150,
        150,  150,  147,  147,  147,  150,  150,  150,  147,  147,
        147,  150,  147,  147,  147,  147,  147,  147,  147,  147,
        147,  147,  147,  147,  147,  147,    0,  147,  147,  147,
        147
                                          } ;

static yyconst flex_int16_t yy_nxt[294] = {   0,
        4,    5,    6,    7,    8,    9,   10,   11,   12,   13,
        14,   15,   16,    8,   17,   18,   19,   20,   21,   17,
        22,   17,   17,   23,   24,   17,   25,   17,   26,   27,
        28,   29,   30,   17,   31,   32,   17,   33,   41,   42,
        44,   45,   46,   48,   49,  147,  147,  147,  147,  147,
        147,  147,  147,  147,  147,  147,  147,   41,   42,   43,
        55,   69,  147,  147,   60,   76,   51,  147,   61,   52,
        54,   57,   56,   59,   58,   53,   62,   64,   66,   65,
        63,  147,   70,  147,  147,   71,  147,  147,  147,   81,
        147,  147,  147,   67,   75,  147,   77,   68,  147,  147,

        147,  147,  147,  147,   83,  147,   82,  147,  147,   79,
        78,   87,  147,   80,   91,   88,   84,   85,   86,   92,
        94,   89,  147,   90,  147,  147,   81,  147,  147,  147,
        102,   93,   95,  147,  147,  147,  147,   97,  147,  147,
        101,  147,  147,  147,   96,  147,  104,  102,   99,  147,
        105,  106,  103,   98,  100,  147,  112,  107,  147,  111,
        113,  108,  147,  110,  147,  147,  109,  147,  117,  147,
        147,  147,  115,  147,  129,  135,  119,  114,  147,  120,
        116,  118,  121,  122,  144,  135,  127,  146,  144,  143,
        126,  147,  128,  142,  141,  140,  132,  139,  137,  136,

        134,  133,  147,  147,  131,  130,  138,  129,  147,  147,
        147,  147,  147,  125,  124,  145,   36,   36,   36,   39,
        39,   39,   50,   50,  123,  147,  147,  147,  147,  147,
        147,  147,  147,  147,   73,  147,  147,   74,  147,   73,
        40,   37,   37,   34,   72,  147,   47,   40,   37,   38,
        37,   35,   34,  147,    3,  147,  147,  147,  147,  147,
        147,  147,  147,  147,  147,  147,  147,  147,  147,  147,
        147,  147,  147,  147,  147,  147,  147,  147,  147,  147,
        147,  147,  147,  147,  147,  147,  147,  147,  147,  147,
        147,  147,  147

                                          } ;

static yyconst flex_int16_t yy_chk[294] = {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,   12,   12,
        13,   13,   13,   15,   15,   18,   19,   20,   21,   22,
        23,   24,   25,   30,   26,   52,   27,   42,   42,  150,
        21,   30,   28,   32,   25,   52,   18,   31,   25,   18,
        20,   23,   22,   24,   23,   19,   26,   27,   28,   27,
        26,   29,   31,   51,   54,   32,   55,   56,   57,   58,
        59,   60,   62,   29,   51,   58,   54,   29,   61,   63,

        65,   66,   67,   68,   60,   69,   59,   70,   71,   56,
        55,   65,   77,   57,   69,   66,   61,   62,   63,   70,
        77,   67,   78,   68,   79,   80,   81,   82,   84,   83,
        85,   71,   78,   86,   87,   88,   85,   80,   94,   89,
        84,   90,   91,   92,   79,   97,   87,  102,   82,  104,
        88,   89,   86,   81,   83,  105,   98,   90,  106,   97,
        98,   91,  107,   94,  108,  117,   92,  109,  106,  122,
        110,  111,  104,  127,  129,  135,  108,  102,  118,  109,
        105,  107,  110,  111,  144,  132,  118,  145,  143,  140,
        117,  132,  122,  139,  138,  137,  127,  136,  134,  133,

        131,  129,  128,  126,  125,  124,  135,  123,  121,  120,
        119,  116,  115,  114,  113,  144,  148,  148,  148,  149,
        149,  149,  151,  151,  112,  103,  101,  100,   99,   96,
        95,   93,   76,   75,   73,   64,   53,   50,   43,   41,
        40,   39,   36,   34,   33,   17,   14,   11,   10,    9,
        7,    6,    5,    3,  147,  147,  147,  147,  147,  147,
        147,  147,  147,  147,  147,  147,  147,  147,  147,  147,
        147,  147,  147,  147,  147,  147,  147,  147,  147,  147,
        147,  147,  147,  147,  147,  147,  147,  147,  147,  147,
        147,  147,  147

                                          } ;

static yy_state_type yy_last_accepting_state;
static char *yy_last_accepting_cpos;

extern int yy_flex_debug;
int yy_flex_debug = 0;

/* The intent behind this definition is that it'll catch
 * any uses of REJECT which flex missed.
 */
#define REJECT reject_used_but_not_detected
#define yymore() yymore_used_but_not_detected
#define YY_MORE_ADJ 0
#define YY_RESTORE_YY_MORE_OFFSET
char *yytext;
#line 1 "sqlscanner.l"
/* This file is part of the KDE project
   Copyright (C) 2004 Lucijan Busch <lucijan@kde.org>
   Copyright (C) 2004-2006 Jarosław Staniek <staniek@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/
#line 22 "sqlscanner.l"
#include <field.h>
#include <expression.h>

#include "sqlparser.h"
#include "sqltypes.h"
#include <iostream>
#include <kdebug.h>
#include <klocale.h>

#define YY_NO_UNPUT
#define ECOUNT current += yyleng; ctoken = yytext

extern void setError(const QString& errDesc);
extern void setError(const QString& errName, const QString& errDesc);

/* *** Please reflect changes to this file in ../driver_p.cpp *** */
/*identifier [a-zA-Z_][a-zA-Z_0-9]* */
/* quoted_identifier (\"[a-zA-Z_0-9]+\") */
/* todo: support for real numbers */
#line 610 "sqlscanner.cpp"

#define INITIAL 0

#ifndef YY_NO_UNISTD_H
/* Special case for "unistd.h", since it is non-ANSI. We include it way
 * down here because we want the user's section 1 to have been scanned first.
 * The user has a chance to override it with an option.
 */
#include <unistd.h>
#endif

#ifndef YY_EXTRA_TYPE
#define YY_EXTRA_TYPE void *
#endif

static int yy_init_globals(void);

/* Macros after this point can all be overridden by user definitions in
 * section 1.
 */

#ifndef YY_SKIP_YYWRAP
#ifdef __cplusplus
extern "C" int yywrap(void);
#else
extern int yywrap(void);
#endif
#endif

static void yyunput(int c, char *buf_ptr);

#ifndef yytext_ptr
static void yy_flex_strncpy(char *, yyconst char *, int);
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen(yyconst char *);
#endif

#ifndef YY_NO_INPUT

#ifdef __cplusplus
static int yyinput(void);
#else
static int input(void);
#endif

#endif

/* Amount of stuff to slurp up with each read. */
#ifndef YY_READ_BUF_SIZE
#define YY_READ_BUF_SIZE 8192
#endif

/* Copy whatever the last rule matched to the standard output. */
#ifndef ECHO
/* This used to be an fputs(), but since the string might contain NUL's,
 * we now use fwrite().
 */
#define ECHO (void) fwrite( yytext, yyleng, 1, yyout )
#endif

/* Gets input and stuffs it into "buf".  number of characters read, or YY_NULL,
 * is returned in "result".
 */
#ifndef YY_INPUT
#define YY_INPUT(buf,result,max_size) \
    if ( YY_CURRENT_BUFFER_LVALUE->yy_is_interactive ) \
    { \
        int c = '*'; \
        size_t n; \
        for ( n = 0; n < (size_t)max_size && \
                (c = getc( yyin )) != EOF && c != '\n'; ++n ) \
            buf[n] = (char) c; \
        if ( c == '\n' ) \
            buf[n++] = (char) c; \
        if ( c == EOF && ferror( yyin ) ) \
            YY_FATAL_ERROR( "input in flex scanner failed" ); \
        result = n; \
    } \
    else \
    { \
        errno=0; \
        while ( (result = fread(buf, 1, max_size, yyin))==0 && ferror(yyin)) \
        { \
            if( errno != EINTR) \
            { \
                YY_FATAL_ERROR( "input in flex scanner failed" ); \
                break; \
            } \
            errno=0; \
            clearerr(yyin); \
        } \
    }\
    \
     
#endif

/* No semi-colon after return; correct usage is to write "yyterminate();" -
 * we don't want an extra ';' after the "return" because that will cause
 * some compilers to complain about unreachable statements.
 */
#ifndef yyterminate
#define yyterminate() return YY_NULL
#endif

/* Number of entries by which start-condition stack grows. */
#ifndef YY_START_STACK_INCR
#define YY_START_STACK_INCR 25
#endif

/* Report a fatal error. */
#ifndef YY_FATAL_ERROR
#define YY_FATAL_ERROR(msg) yy_fatal_error( msg )
#endif

/* end tables serialization structures and prototypes */

/* Default declaration of generated scanner - a define so the user can
 * easily add parameters.
 */
#ifndef YY_DECL
#define YY_DECL_IS_OURS 1

extern int yylex(void);

#define YY_DECL int yylex (void)
#endif /* !YY_DECL */

/* Code executed at the beginning of each rule, after yytext and yyleng
 * have been set up.
 */
#ifndef YY_USER_ACTION
#define YY_USER_ACTION
#endif

/* Code executed at the end of each rule. */
#ifndef YY_BREAK
#define YY_BREAK break;
#endif

#define YY_RULE_SETUP \
    YY_USER_ACTION

/** The main scanner function which does all the work.
 */
YY_DECL {
    register yy_state_type yy_current_state;
    register char *yy_cp, *yy_bp;
    register int yy_act;

#line 58 "sqlscanner.l"



#line 767 "sqlscanner.cpp"

    if (!(yy_init)) {
        (yy_init) = 1;

#ifdef YY_USER_INIT
        YY_USER_INIT;
#endif

        if (!(yy_start))
            (yy_start) = 1; /* first start state */

        if (! yyin)
            yyin = stdin;

        if (! yyout)
            yyout = stdout;

        if (! YY_CURRENT_BUFFER) {
            yyensure_buffer_stack();
            YY_CURRENT_BUFFER_LVALUE =
                yy_create_buffer(yyin, YY_BUF_SIZE);
        }

        yy_load_buffer_state();
    }

    while (1) {  /* loops until end-of-file is reached */
        yy_cp = (yy_c_buf_p);

        /* Support of yytext. */
        *yy_cp = (yy_hold_char);

        /* yy_bp points to the position in yy_ch_buf of the start of
         * the current run.
         */
        yy_bp = yy_cp;

        yy_current_state = (yy_start);
    yy_match:
        do {
            register YY_CHAR yy_c = yy_ec[YY_SC_TO_UI(*yy_cp)];
            if (yy_accept[yy_current_state]) {
                (yy_last_accepting_state) = yy_current_state;
                (yy_last_accepting_cpos) = yy_cp;
            }
            while (yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state) {
                yy_current_state = (int) yy_def[yy_current_state];
                if (yy_current_state >= 148)
                    yy_c = yy_meta[(unsigned int) yy_c];
            }
            yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
            ++yy_cp;
        } while (yy_current_state != 147);
        yy_cp = (yy_last_accepting_cpos);
        yy_current_state = (yy_last_accepting_state);

    yy_find_action:
        yy_act = yy_accept[yy_current_state];

        YY_DO_BEFORE_ACTION;

    do_action: /* This label is used only to access EOF actions. */

        switch (yy_act) { /* beginning of action switch */
        case 0: /* must back up */
            /* undo the effects of YY_DO_BEFORE_ACTION */
            *yy_cp = (yy_hold_char);
            yy_cp = (yy_last_accepting_cpos);
            yy_current_state = (yy_last_accepting_state);
            goto yy_find_action;

        case 1:
            YY_RULE_SETUP
#line 61 "sqlscanner.l"
            {
                ECOUNT;
                return NOT_EQUAL;
            }
            YY_BREAK
        case 2:
            YY_RULE_SETUP
#line 66 "sqlscanner.l"
            {
                ECOUNT;
                return NOT_EQUAL2;
            }
            YY_BREAK
        case 3:
            YY_RULE_SETUP
#line 71 "sqlscanner.l"
            {
                ECOUNT;
                return '=';
            }
            YY_BREAK
        case 4:
            YY_RULE_SETUP
#line 76 "sqlscanner.l"
            {
                ECOUNT;
                return LESS_OR_EQUAL;
            }
            YY_BREAK
        case 5:
            YY_RULE_SETUP
#line 81 "sqlscanner.l"
            {
                ECOUNT;
                return GREATER_OR_EQUAL;
            }
            YY_BREAK
        case 6:
            YY_RULE_SETUP
#line 86 "sqlscanner.l"
            {
                ECOUNT;
                return SQL_IN;
            }
            YY_BREAK
        case 7:
            YY_RULE_SETUP
#line 91 "sqlscanner.l"
            {
//TODO: what about hex or octal values?
                //we're using QString:toLongLong() here because atoll() is not so portable:
                ECOUNT;
                bool ok;
                yylval.integerValue = QString(yytext).toLongLong(&ok);
                if (!ok) {
                    setError(i18n("Invalid integer number"), i18n("This integer number may be too large."));
                    return SCAN_ERROR;
                }
// yylval.integerValue = atol(yytext);
                return INTEGER_CONST;
            }
            YY_BREAK
        case 8:
            YY_RULE_SETUP
#line 105 "sqlscanner.l"
            {
                char *p = yytext;
                if (yytext[0] == '.') { /* no integer part */
                    yylval.realValue.integer = 0;
                } else {
                    yylval.realValue.integer = atoi(p);
                    int i = 0;
                    while (p && i < yyleng && *p != '.') {
                        i++;
                        p++;
                    }
                    if (i == 0 || !p || *p != '.') {
                        yylval.realValue.fractional = 0;
                        return REAL_CONST;
                    }
                }
                /* fractional part */
                p++;
                yylval.realValue.fractional = atoi(p);
                return REAL_CONST;
            }
            YY_BREAK
        case 9:
            YY_RULE_SETUP
#line 128 "sqlscanner.l"
            {
                ECOUNT;
                return AND;
            }
            YY_BREAK
        case 10:
            YY_RULE_SETUP
#line 133 "sqlscanner.l"
            {
                ECOUNT;
                return AS;
            }
            YY_BREAK
        case 11:
            YY_RULE_SETUP
#line 138 "sqlscanner.l"
            {
                ECOUNT;
                return CREATE;
            }
            YY_BREAK
        case 12:
            YY_RULE_SETUP
#line 143 "sqlscanner.l"
            {
                ECOUNT;
                return FROM;
            }
            YY_BREAK
        case 13:
            YY_RULE_SETUP
#line 148 "sqlscanner.l"
            {
                ECOUNT;
                return SQL_TYPE;
            }
            YY_BREAK
        case 14:
            YY_RULE_SETUP
#line 153 "sqlscanner.l"
            {
                ECOUNT;
                return JOIN;
            }
            YY_BREAK
        case 15:
            YY_RULE_SETUP
#line 158 "sqlscanner.l"
            {
                ECOUNT;
                return LEFT;
            }
            YY_BREAK
        case 16:
            YY_RULE_SETUP
#line 163 "sqlscanner.l"
            {
                ECOUNT;
                return LIKE;
            }
            YY_BREAK
        case 17:
            /* rule 17 can match eol */
            YY_RULE_SETUP
#line 168 "sqlscanner.l"
            {
                ECOUNT;
                return NOT_SIMILAR_TO;
            }
            YY_BREAK
        case 18:
            /* rule 18 can match eol */
            YY_RULE_SETUP
#line 173 "sqlscanner.l"
            {
                ECOUNT;
                return SIMILAR_TO;
            }
            YY_BREAK
        case 19:
            /* rule 19 can match eol */
            YY_RULE_SETUP
#line 178 "sqlscanner.l"
            {
                ECOUNT;
                return SQL_IS_NOT_NULL;
            }
            YY_BREAK
        case 20:
            /* rule 20 can match eol */
            YY_RULE_SETUP
#line 183 "sqlscanner.l"
            {
                ECOUNT;
                return SQL_IS_NULL;
            }
            YY_BREAK
        case 21:
            YY_RULE_SETUP
#line 188 "sqlscanner.l"
            {
                ECOUNT;
                return NOT;
            }
            YY_BREAK
        case 22:
            YY_RULE_SETUP
#line 193 "sqlscanner.l"
            {
                ECOUNT;
                return SQL_IS;
            }
            YY_BREAK
        case 23:
            YY_RULE_SETUP
#line 198 "sqlscanner.l"
            {
                ECOUNT;
                return SQL_NULL;
            }
            YY_BREAK
        case 24:
            YY_RULE_SETUP
#line 203 "sqlscanner.l"
            {
                ECOUNT;
                return SQL_ON;
            }
            YY_BREAK
        case 25:
            YY_RULE_SETUP
#line 208 "sqlscanner.l"
            {
                ECOUNT;
                return OR;
            }
            YY_BREAK
        case 26:
            YY_RULE_SETUP
#line 213 "sqlscanner.l"
            { /* also means OR for numbers (mysql) */
                ECOUNT;
                return CONCATENATION;
            }
            YY_BREAK
        case 27:
            YY_RULE_SETUP
#line 218 "sqlscanner.l"
            {
                ECOUNT;
                return BITWISE_SHIFT_LEFT;
            }
            YY_BREAK
        case 28:
            YY_RULE_SETUP
#line 223 "sqlscanner.l"
            {
                ECOUNT;
                return BITWISE_SHIFT_RIGHT;
            }
            YY_BREAK
        case 29:
            YY_RULE_SETUP
#line 228 "sqlscanner.l"
            {
                ECOUNT;
                return XOR;
            }
            YY_BREAK
        case 30:
            YY_RULE_SETUP
#line 233 "sqlscanner.l"
            {
                ECOUNT;
                return RIGHT;
            }
            YY_BREAK
        case 31:
            YY_RULE_SETUP
#line 238 "sqlscanner.l"
            {
                ECOUNT;
                return SELECT;
            }
            YY_BREAK
        case 32:
            YY_RULE_SETUP
#line 243 "sqlscanner.l"
            {
                ECOUNT;
                return TABLE;
            }
            YY_BREAK
        case 33:
            YY_RULE_SETUP
#line 248 "sqlscanner.l"
            {
                ECOUNT;
                return WHERE;
            }
            YY_BREAK
        case 34:
            YY_RULE_SETUP
#line 253 "sqlscanner.l"
            {
                ECOUNT;
                return ORDER;
            }
            YY_BREAK
        case 35:
            YY_RULE_SETUP
#line 258 "sqlscanner.l"
            {
                ECOUNT;
                return BY;
            }
            YY_BREAK
        case 36:
            YY_RULE_SETUP
#line 263 "sqlscanner.l"
            {
                ECOUNT;
                return ASC;
            }
            YY_BREAK
        case 37:
            YY_RULE_SETUP
#line 268 "sqlscanner.l"
            {
                ECOUNT;
                return DESC;
            }
            YY_BREAK
        case 38:
            /* rule 38 can match eol */
            YY_RULE_SETUP
#line 273 "sqlscanner.l"
            {
                ECOUNT;
                yylval.stringValue = new QString(QString::fromUtf8(yytext + 1, yyleng - 2));
                return CHARACTER_STRING_LITERAL;

                /* "ZZZ" sentinel for script */
            }
            YY_BREAK
        case 39:
            YY_RULE_SETUP
#line 281 "sqlscanner.l"
            {
                KexiDBDbg << "yytext: '" << yytext << "' (" << yyleng << ")";
                ECOUNT;
                yylval.stringValue = new QString(QString::fromUtf8(yytext, yyleng));
                if (yytext[0] >= '0' && yytext[0] <= '9') {
                    setError(i18n("Invalid identifier"),
                             i18n("Identifiers should start with a letter or '_' character"));
                    return SCAN_ERROR;
                }
                return IDENTIFIER;
            }
            YY_BREAK
        case 40:
            /* rule 40 can match eol */
            YY_RULE_SETUP
#line 293 "sqlscanner.l"
            {
                KexiDBDbg << "yytext: '" << yytext << "' (" << yyleng << ")";
                ECOUNT;
                yylval.stringValue = new QString(QString::fromUtf8(yytext + 1, yyleng - 2));
                return QUERY_PARAMETER;
            }
            YY_BREAK
        case 41:
            /* rule 41 can match eol */
            YY_RULE_SETUP
#line 300 "sqlscanner.l"
            {
                ECOUNT;
            }
            YY_BREAK
        case 42:
            YY_RULE_SETUP
#line 304 "sqlscanner.l"
            {
                KexiDBDbg << "char: '" << yytext[0] << "'";
                ECOUNT;
                return yytext[0];
            }
            YY_BREAK
        case 43:
            YY_RULE_SETUP
#line 310 "sqlscanner.l"
            ECHO;
            YY_BREAK
#line 1233 "sqlscanner.cpp"
        case YY_STATE_EOF(INITIAL):
            yyterminate();

        case YY_END_OF_BUFFER: {
            /* Amount of text matched not including the EOB char. */
            int yy_amount_of_matched_text = (int)(yy_cp - (yytext_ptr)) - 1;

            /* Undo the effects of YY_DO_BEFORE_ACTION. */
            *yy_cp = (yy_hold_char);
            YY_RESTORE_YY_MORE_OFFSET

            if (YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_NEW) {
                /* We're scanning a new file or input source.  It's
                 * possible that this happened because the user
                 * just pointed yyin at a new source and called
                 * yylex().  If so, then we have to assure
                 * consistency between YY_CURRENT_BUFFER and our
                 * globals.  Here is the right place to do so, because
                 * this is the first action (other than possibly a
                 * back-up) that will match for the new input source.
                 */
                (yy_n_chars) = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
                YY_CURRENT_BUFFER_LVALUE->yy_input_file = yyin;
                YY_CURRENT_BUFFER_LVALUE->yy_buffer_status = YY_BUFFER_NORMAL;
            }

            /* Note that here we test for yy_c_buf_p "<=" to the position
             * of the first EOB in the buffer, since yy_c_buf_p will
             * already have been incremented past the NUL character
             * (since all states make transitions on EOB to the
             * end-of-buffer state).  Contrast this with the test
             * in input().
             */
            if ((yy_c_buf_p) <= &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)]) { /* This was really a NUL. */
                yy_state_type yy_next_state;

                (yy_c_buf_p) = (yytext_ptr) + yy_amount_of_matched_text;

                yy_current_state = yy_get_previous_state();

                /* Okay, we're now positioned to make the NUL
                 * transition.  We couldn't have
                 * yy_get_previous_state() go ahead and do it
                 * for us because it doesn't know how to deal
                 * with the possibility of jamming (and we don't
                 * want to build jamming into it because then it
                 * will run more slowly).
                 */

                yy_next_state = yy_try_NUL_trans(yy_current_state);

                yy_bp = (yytext_ptr) + YY_MORE_ADJ;

                if (yy_next_state) {
                    /* Consume the NUL. */
                    yy_cp = ++(yy_c_buf_p);
                    yy_current_state = yy_next_state;
                    goto yy_match;
                }

                else {
                    yy_cp = (yy_last_accepting_cpos);
                    yy_current_state = (yy_last_accepting_state);
                    goto yy_find_action;
                }
            }

            else switch (yy_get_next_buffer()) {
                case EOB_ACT_END_OF_FILE: {
                    (yy_did_buffer_switch_on_eof) = 0;

                    if (yywrap()) {
                        /* Note: because we've taken care in
                         * yy_get_next_buffer() to have set up
                         * yytext, we can now set up
                         * yy_c_buf_p so that if some total
                         * hoser (like flex itself) wants to
                         * call the scanner after we return the
                         * YY_NULL, it'll still work - another
                         * YY_NULL will get returned.
                         */
                        (yy_c_buf_p) = (yytext_ptr) + YY_MORE_ADJ;

                        yy_act = YY_STATE_EOF(YY_START);
                        goto do_action;
                    }

                    else {
                        if (!(yy_did_buffer_switch_on_eof))
                            YY_NEW_FILE;
                    }
                    break;
                }

                case EOB_ACT_CONTINUE_SCAN:
                    (yy_c_buf_p) =
                        (yytext_ptr) + yy_amount_of_matched_text;

                    yy_current_state = yy_get_previous_state();

                    yy_cp = (yy_c_buf_p);
                    yy_bp = (yytext_ptr) + YY_MORE_ADJ;
                    goto yy_match;

                case EOB_ACT_LAST_MATCH:
                    (yy_c_buf_p) =
                        &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)];

                    yy_current_state = yy_get_previous_state();

                    yy_cp = (yy_c_buf_p);
                    yy_bp = (yytext_ptr) + YY_MORE_ADJ;
                    goto yy_find_action;
                }
            break;
        }

        default:
            YY_FATAL_ERROR(
                "fatal flex scanner internal error--no action found");
        } /* end of action switch */
    } /* end of scanning one token */
} /* end of yylex */

/* yy_get_next_buffer - try to read in a new buffer
 *
 * Returns a code representing an action:
 * EOB_ACT_LAST_MATCH -
 * EOB_ACT_CONTINUE_SCAN - continue scanning from current position
 * EOB_ACT_END_OF_FILE - end of file
 */
static int yy_get_next_buffer(void)
{
    register char *dest = YY_CURRENT_BUFFER_LVALUE->yy_ch_buf;
    register char *source = (yytext_ptr);
    register int number_to_move, i;
    int ret_val;

    if ((yy_c_buf_p) > &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars) + 1])
        YY_FATAL_ERROR(
            "fatal flex scanner internal error--end of buffer missed");

    if (YY_CURRENT_BUFFER_LVALUE->yy_fill_buffer == 0) { /* Don't try to fill the buffer, so this is an EOF. */
        if ((yy_c_buf_p) - (yytext_ptr) - YY_MORE_ADJ == 1) {
            /* We matched a single character, the EOB, so
             * treat this as a final EOF.
             */
            return EOB_ACT_END_OF_FILE;
        }

        else {
            /* We matched some text prior to the EOB, first
             * process it.
             */
            return EOB_ACT_LAST_MATCH;
        }
    }

    /* Try to read more data. */

    /* First move last chars to start of buffer. */
    number_to_move = (int)((yy_c_buf_p) - (yytext_ptr)) - 1;

    for (i = 0; i < number_to_move; ++i)
        *(dest++) = *(source++);

    if (YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_EOF_PENDING)
        /* don't do the read, it's not guaranteed to return an EOF,
         * just force an EOF
         */
        YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars) = 0;

    else {
        int num_to_read =
            YY_CURRENT_BUFFER_LVALUE->yy_buf_size - number_to_move - 1;

        while (num_to_read <= 0) { /* Not enough room in the buffer - grow it. */

            /* just a shorter name for the current buffer */
            YY_BUFFER_STATE b = YY_CURRENT_BUFFER;

            int yy_c_buf_p_offset =
                (int)((yy_c_buf_p) - b->yy_ch_buf);

            if (b->yy_is_our_buffer) {
                int new_size = b->yy_buf_size * 2;

                if (new_size <= 0)
                    b->yy_buf_size += b->yy_buf_size / 8;
                else
                    b->yy_buf_size *= 2;

                b->yy_ch_buf = (char *)
                               /* Include room in for 2 EOB chars. */
                               yyrealloc((void *) b->yy_ch_buf, b->yy_buf_size + 2);
            } else
                /* Can't grow it, we don't own it. */
                b->yy_ch_buf = 0;

            if (! b->yy_ch_buf)
                YY_FATAL_ERROR(
                    "fatal error - scanner input buffer overflow");

            (yy_c_buf_p) = &b->yy_ch_buf[yy_c_buf_p_offset];

            num_to_read = YY_CURRENT_BUFFER_LVALUE->yy_buf_size -
                          number_to_move - 1;

        }

        if (num_to_read > YY_READ_BUF_SIZE)
            num_to_read = YY_READ_BUF_SIZE;

        /* Read in more data. */
        YY_INPUT((&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[number_to_move]),
                 (yy_n_chars), num_to_read);

        YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
    }

    if ((yy_n_chars) == 0) {
        if (number_to_move == YY_MORE_ADJ) {
            ret_val = EOB_ACT_END_OF_FILE;
            yyrestart(yyin);
        }

        else {
            ret_val = EOB_ACT_LAST_MATCH;
            YY_CURRENT_BUFFER_LVALUE->yy_buffer_status =
                YY_BUFFER_EOF_PENDING;
        }
    }

    else
        ret_val = EOB_ACT_CONTINUE_SCAN;

    (yy_n_chars) += number_to_move;
    YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)] = YY_END_OF_BUFFER_CHAR;
    YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars) + 1] = YY_END_OF_BUFFER_CHAR;

    (yytext_ptr) = &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[0];

    return ret_val;
}

/* yy_get_previous_state - get the state just before the EOB char was reached */

static yy_state_type yy_get_previous_state(void)
{
    register yy_state_type yy_current_state;
    register char *yy_cp;

    yy_current_state = (yy_start);

    for (yy_cp = (yytext_ptr) + YY_MORE_ADJ; yy_cp < (yy_c_buf_p); ++yy_cp) {
        register YY_CHAR yy_c = (*yy_cp ? yy_ec[YY_SC_TO_UI(*yy_cp)] : 1);
        if (yy_accept[yy_current_state]) {
            (yy_last_accepting_state) = yy_current_state;
            (yy_last_accepting_cpos) = yy_cp;
        }
        while (yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state) {
            yy_current_state = (int) yy_def[yy_current_state];
            if (yy_current_state >= 148)
                yy_c = yy_meta[(unsigned int) yy_c];
        }
        yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
    }

    return yy_current_state;
}

/* yy_try_NUL_trans - try to make a transition on the NUL character
 *
 * synopsis
 * next_state = yy_try_NUL_trans( current_state );
 */
static yy_state_type yy_try_NUL_trans(yy_state_type yy_current_state)
{
    register int yy_is_jam;
    register char *yy_cp = (yy_c_buf_p);

    register YY_CHAR yy_c = 1;
    if (yy_accept[yy_current_state]) {
        (yy_last_accepting_state) = yy_current_state;
        (yy_last_accepting_cpos) = yy_cp;
    }
    while (yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state) {
        yy_current_state = (int) yy_def[yy_current_state];
        if (yy_current_state >= 148)
            yy_c = yy_meta[(unsigned int) yy_c];
    }
    yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
    yy_is_jam = (yy_current_state == 147);

    return yy_is_jam ? 0 : yy_current_state;
}

static void yyunput(int c, register char * yy_bp)
{
    register char *yy_cp;

    yy_cp = (yy_c_buf_p);

    /* undo effects of setting up yytext */
    *yy_cp = (yy_hold_char);

    if (yy_cp < YY_CURRENT_BUFFER_LVALUE->yy_ch_buf + 2) { /* need to shift things up to make room */
        /* +2 for EOB chars. */
        register int number_to_move = (yy_n_chars) + 2;
        register char *dest = &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[
                                  YY_CURRENT_BUFFER_LVALUE->yy_buf_size + 2];
        register char *source =
            &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[number_to_move];

        while (source > YY_CURRENT_BUFFER_LVALUE->yy_ch_buf)
            *--dest = *--source;

        yy_cp += (int)(dest - source);
        yy_bp += (int)(dest - source);
        YY_CURRENT_BUFFER_LVALUE->yy_n_chars =
            (yy_n_chars) = YY_CURRENT_BUFFER_LVALUE->yy_buf_size;

        if (yy_cp < YY_CURRENT_BUFFER_LVALUE->yy_ch_buf + 2)
            YY_FATAL_ERROR("flex scanner push-back overflow");
    }

    *--yy_cp = (char) c;

    (yytext_ptr) = yy_bp;
    (yy_hold_char) = *yy_cp;
    (yy_c_buf_p) = yy_cp;
}

#ifndef YY_NO_INPUT
#ifdef __cplusplus
static int yyinput(void)
#else
static int input(void)
#endif

{
    int c;

    *(yy_c_buf_p) = (yy_hold_char);

    if (*(yy_c_buf_p) == YY_END_OF_BUFFER_CHAR) {
        /* yy_c_buf_p now points to the character we want to return.
         * If this occurs *before* the EOB characters, then it's a
         * valid NUL; if not, then we've hit the end of the buffer.
         */
        if ((yy_c_buf_p) < &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)])
            /* This was really a NUL. */
            *(yy_c_buf_p) = '\0';

        else { /* need more input */
            int offset = (yy_c_buf_p) - (yytext_ptr);
            ++(yy_c_buf_p);

            switch (yy_get_next_buffer()) {
            case EOB_ACT_LAST_MATCH:
                /* This happens because yy_g_n_b()
                 * sees that we've accumulated a
                 * token and flags that we need to
                 * try matching the token before
                 * proceeding.  But for input(),
                 * there's no matching to consider.
                 * So convert the EOB_ACT_LAST_MATCH
                 * to EOB_ACT_END_OF_FILE.
                 */

                /* Reset buffer status. */
                yyrestart(yyin);

                /*FALLTHROUGH*/

            case EOB_ACT_END_OF_FILE: {
                if (yywrap())
                    return EOF;

                if (!(yy_did_buffer_switch_on_eof))
                    YY_NEW_FILE;
#ifdef __cplusplus
                return yyinput();
#else
                return input();
#endif
            }

            case EOB_ACT_CONTINUE_SCAN:
                (yy_c_buf_p) = (yytext_ptr) + offset;
                break;
            }
        }
    }

    c = *(unsigned char *)(yy_c_buf_p);  /* cast for 8-bit char's */
    *(yy_c_buf_p) = '\0'; /* preserve yytext */
    (yy_hold_char) = *++(yy_c_buf_p);

    return c;
}
#endif /* ifndef YY_NO_INPUT */

/** Immediately switch to a different input stream.
 * @param input_file A readable stream.
 *
 * @note This function does not reset the start condition to @c INITIAL .
 */
void yyrestart(FILE * input_file)
{

    if (! YY_CURRENT_BUFFER) {
        yyensure_buffer_stack();
        YY_CURRENT_BUFFER_LVALUE =
            yy_create_buffer(yyin, YY_BUF_SIZE);
    }

    yy_init_buffer(YY_CURRENT_BUFFER, input_file);
    yy_load_buffer_state();
}

/** Switch to a different input buffer.
 * @param new_buffer The new input buffer.
 *
 */
void yy_switch_to_buffer(YY_BUFFER_STATE  new_buffer)
{

    /* TODO. We should be able to replace this entire function body
     * with
     *  yypop_buffer_state();
     *  yypush_buffer_state(new_buffer);
       */
    yyensure_buffer_stack();
    if (YY_CURRENT_BUFFER == new_buffer)
        return;

    if (YY_CURRENT_BUFFER) {
        /* Flush out information for old buffer. */
        *(yy_c_buf_p) = (yy_hold_char);
        YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = (yy_c_buf_p);
        YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
    }

    YY_CURRENT_BUFFER_LVALUE = new_buffer;
    yy_load_buffer_state();

    /* We don't actually know whether we did this switch during
     * EOF (yywrap()) processing, but the only time this flag
     * is looked at is after yywrap() is called, so it's safe
     * to go ahead and always set it.
     */
    (yy_did_buffer_switch_on_eof) = 1;
}

static void yy_load_buffer_state(void)
{
    (yy_n_chars) = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
    (yytext_ptr) = (yy_c_buf_p) = YY_CURRENT_BUFFER_LVALUE->yy_buf_pos;
    yyin = YY_CURRENT_BUFFER_LVALUE->yy_input_file;
    (yy_hold_char) = *(yy_c_buf_p);
}

/** Allocate and initialize an input buffer state.
 * @param file A readable stream.
 * @param size The character buffer size in bytes. When in doubt, use @c YY_BUF_SIZE.
 *
 * @return the allocated buffer state.
 */
YY_BUFFER_STATE yy_create_buffer(FILE * file, int  size)
{
    YY_BUFFER_STATE b;

    b = (YY_BUFFER_STATE) yyalloc(sizeof(struct yy_buffer_state));
    if (! b)
        YY_FATAL_ERROR("out of dynamic memory in yy_create_buffer()");

    b->yy_buf_size = size;

    /* yy_ch_buf has to be 2 characters longer than the size given because
     * we need to put in 2 end-of-buffer characters.
     */
    b->yy_ch_buf = (char *) yyalloc(b->yy_buf_size + 2);
    if (! b->yy_ch_buf)
        YY_FATAL_ERROR("out of dynamic memory in yy_create_buffer()");

    b->yy_is_our_buffer = 1;

    yy_init_buffer(b, file);

    return b;
}

/** Destroy the buffer.
 * @param b a buffer created with yy_create_buffer()
 *
 */
void yy_delete_buffer(YY_BUFFER_STATE  b)
{

    if (! b)
        return;

    if (b == YY_CURRENT_BUFFER)   /* Not sure if we should pop here. */
        YY_CURRENT_BUFFER_LVALUE = (YY_BUFFER_STATE) 0;

    if (b->yy_is_our_buffer)
        yyfree((void *) b->yy_ch_buf);

    yyfree((void *) b);
}

/* Initializes or reinitializes a buffer.
 * This function is sometimes called more than once on the same buffer,
 * such as during a yyrestart() or at EOF.
 */
static void yy_init_buffer(YY_BUFFER_STATE  b, FILE * file)

{
    int oerrno = errno;

    yy_flush_buffer(b);

    b->yy_input_file = file;
    b->yy_fill_buffer = 1;

    /* If b is the current buffer, then yy_init_buffer was _probably_
     * called from yyrestart() or through yy_get_next_buffer.
     * In that case, we don't want to reset the lineno or column.
     */
    if (b != YY_CURRENT_BUFFER) {
        b->yy_bs_lineno = 1;
        b->yy_bs_column = 0;
    }

    b->yy_is_interactive = 0;

    errno = oerrno;
}

/** Discard all buffered characters. On the next scan, YY_INPUT will be called.
 * @param b the buffer state to be flushed, usually @c YY_CURRENT_BUFFER.
 *
 */
void yy_flush_buffer(YY_BUFFER_STATE  b)
{
    if (! b)
        return;

    b->yy_n_chars = 0;

    /* We always need two end-of-buffer characters.  The first causes
     * a transition to the end-of-buffer state.  The second causes
     * a jam in that state.
     */
    b->yy_ch_buf[0] = YY_END_OF_BUFFER_CHAR;
    b->yy_ch_buf[1] = YY_END_OF_BUFFER_CHAR;

    b->yy_buf_pos = &b->yy_ch_buf[0];

    b->yy_at_bol = 1;
    b->yy_buffer_status = YY_BUFFER_NEW;

    if (b == YY_CURRENT_BUFFER)
        yy_load_buffer_state();
}

/** Pushes the new state onto the stack. The new state becomes
 *  the current state. This function will allocate the stack
 *  if necessary.
 *  @param new_buffer The new state.
 *
 */
void yypush_buffer_state(YY_BUFFER_STATE new_buffer)
{
    if (new_buffer == NULL)
        return;

    yyensure_buffer_stack();

    /* This block is copied from yy_switch_to_buffer. */
    if (YY_CURRENT_BUFFER) {
        /* Flush out information for old buffer. */
        *(yy_c_buf_p) = (yy_hold_char);
        YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = (yy_c_buf_p);
        YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
    }

    /* Only push if top exists. Otherwise, replace top. */
    if (YY_CURRENT_BUFFER)
        (yy_buffer_stack_top)++;
    YY_CURRENT_BUFFER_LVALUE = new_buffer;

    /* copied from yy_switch_to_buffer. */
    yy_load_buffer_state();
    (yy_did_buffer_switch_on_eof) = 1;
}

/** Removes and deletes the top of the stack, if present.
 *  The next element becomes the new top.
 *
 */
void yypop_buffer_state(void)
{
    if (!YY_CURRENT_BUFFER)
        return;

    yy_delete_buffer(YY_CURRENT_BUFFER);
    YY_CURRENT_BUFFER_LVALUE = NULL;
    if ((yy_buffer_stack_top) > 0)
        --(yy_buffer_stack_top);

    if (YY_CURRENT_BUFFER) {
        yy_load_buffer_state();
        (yy_did_buffer_switch_on_eof) = 1;
    }
}

/* Allocates the stack if it does not exist.
 *  Guarantees space for at least one push.
 */
static void yyensure_buffer_stack(void)
{
    int num_to_alloc;

    if (!(yy_buffer_stack)) {

        /* First allocation is just for 2 elements, since we don't know if this
         * scanner will even need a stack. We use 2 instead of 1 to avoid an
         * immediate realloc on the next call.
             */
        num_to_alloc = 1;
        (yy_buffer_stack) = (struct yy_buffer_state**)yyalloc
                            (num_to_alloc * sizeof(struct yy_buffer_state*)
                            );

        memset((yy_buffer_stack), 0, num_to_alloc * sizeof(struct yy_buffer_state*));

        (yy_buffer_stack_max) = num_to_alloc;
        (yy_buffer_stack_top) = 0;
        return;
    }

    if ((yy_buffer_stack_top) >= ((yy_buffer_stack_max)) - 1) {

        /* Increase the buffer to prepare for a possible push. */
        int grow_size = 8 /* arbitrary grow size */;

        num_to_alloc = (yy_buffer_stack_max) + grow_size;
        (yy_buffer_stack) = (struct yy_buffer_state**)yyrealloc
                            ((yy_buffer_stack),
                             num_to_alloc * sizeof(struct yy_buffer_state*)
                            );

        /* zero only the new slots.*/
        memset((yy_buffer_stack) + (yy_buffer_stack_max), 0, grow_size * sizeof(struct yy_buffer_state*));
        (yy_buffer_stack_max) = num_to_alloc;
    }
}

/** Setup the input buffer state to scan directly from a user-specified character buffer.
 * @param base the character buffer
 * @param size the size in bytes of the character buffer
 *
 * @return the newly allocated buffer state object.
 */
YY_BUFFER_STATE yy_scan_buffer(char * base, yy_size_t  size)
{
    YY_BUFFER_STATE b;

    if (size < 2 ||
            base[size-2] != YY_END_OF_BUFFER_CHAR ||
            base[size-1] != YY_END_OF_BUFFER_CHAR)
        /* They forgot to leave room for the EOB's. */
        return 0;

    b = (YY_BUFFER_STATE) yyalloc(sizeof(struct yy_buffer_state));
    if (! b)
        YY_FATAL_ERROR("out of dynamic memory in yy_scan_buffer()");

    b->yy_buf_size = size - 2; /* "- 2" to take care of EOB's */
    b->yy_buf_pos = b->yy_ch_buf = base;
    b->yy_is_our_buffer = 0;
    b->yy_input_file = 0;
    b->yy_n_chars = b->yy_buf_size;
    b->yy_is_interactive = 0;
    b->yy_at_bol = 1;
    b->yy_fill_buffer = 0;
    b->yy_buffer_status = YY_BUFFER_NEW;

    yy_switch_to_buffer(b);

    return b;
}

/** Setup the input buffer state to scan a string. The next call to yylex() will
 * scan from a @e copy of @a str.
 * @param str a NUL-terminated string to scan
 *
 * @return the newly allocated buffer state object.
 * @note If you want to scan bytes that may contain NUL values, then use
 *       yy_scan_bytes() instead.
 */
YY_BUFFER_STATE yy_scan_string(yyconst char * yystr)
{

    return yy_scan_bytes(yystr, strlen(yystr));
}

/** Setup the input buffer state to scan the given bytes. The next call to yylex() will
 * scan from a @e copy of @a bytes.
 * @param bytes the byte buffer to scan
 * @param len the number of bytes in the buffer pointed to by @a bytes.
 *
 * @return the newly allocated buffer state object.
 */
YY_BUFFER_STATE yy_scan_bytes(yyconst char * yybytes, int  _yybytes_len)
{
    YY_BUFFER_STATE b;
    char *buf;
    yy_size_t n;
    int i;

    /* Get memory for full buffer, including space for trailing EOB's. */
    n = _yybytes_len + 2;
    buf = (char *) yyalloc(n);
    if (! buf)
        YY_FATAL_ERROR("out of dynamic memory in yy_scan_bytes()");

    for (i = 0; i < _yybytes_len; ++i)
        buf[i] = yybytes[i];

    buf[_yybytes_len] = buf[_yybytes_len+1] = YY_END_OF_BUFFER_CHAR;

    b = yy_scan_buffer(buf, n);
    if (! b)
        YY_FATAL_ERROR("bad buffer in yy_scan_bytes()");

    /* It's okay to grow etc. this buffer, and we should throw it
     * away when we're done.
     */
    b->yy_is_our_buffer = 1;

    return b;
}

#ifndef YY_EXIT_FAILURE
#define YY_EXIT_FAILURE 2
#endif

static void yy_fatal_error(yyconst char* msg)
{
    (void) fprintf(stderr, "%s\n", msg);
    exit(YY_EXIT_FAILURE);
}

/* Redefine yyless() so it works in section 3 code. */

#undef yyless
#define yyless(n) \
    do \
    { \
        /* Undo effects of setting up yytext. */ \
        int yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
        yytext[yyleng] = (yy_hold_char); \
        (yy_c_buf_p) = yytext + yyless_macro_arg; \
        (yy_hold_char) = *(yy_c_buf_p); \
        *(yy_c_buf_p) = '\0'; \
        yyleng = yyless_macro_arg; \
    } \
    while ( 0 )

/* Accessor  methods (get/set functions) to struct members. */

/** Get the current line number.
 *
 */
int yyget_lineno(void)
{

    return yylineno;
}

/** Get the input stream.
 *
 */
FILE *yyget_in(void)
{
    return yyin;
}

/** Get the output stream.
 *
 */
FILE *yyget_out(void)
{
    return yyout;
}

/** Get the length of the current token.
 *
 */
int yyget_leng(void)
{
    return yyleng;
}

/** Get the current token.
 *
 */

char *yyget_text(void)
{
    return yytext;
}

/** Set the current line number.
 * @param line_number
 *
 */
void yyset_lineno(int  line_number)
{

    yylineno = line_number;
}

/** Set the input stream. This does not discard the current
 * input buffer.
 * @param in_str A readable stream.
 *
 * @see yy_switch_to_buffer
 */
void yyset_in(FILE *  in_str)
{
    yyin = in_str ;
}

void yyset_out(FILE *  out_str)
{
    yyout = out_str ;
}

int yyget_debug(void)
{
    return yy_flex_debug;
}

void yyset_debug(int  bdebug)
{
    yy_flex_debug = bdebug ;
}

static int yy_init_globals(void)
{
    /* Initialization is the same as for the non-reentrant scanner.
    * This function is called from yylex_destroy(), so don't allocate here.
    */

    (yy_buffer_stack) = 0;
    (yy_buffer_stack_top) = 0;
    (yy_buffer_stack_max) = 0;
    (yy_c_buf_p) = (char *) 0;
    (yy_init) = 0;
    (yy_start) = 0;

    /* Defined in main.c */
#ifdef YY_STDINIT
    yyin = stdin;
    yyout = stdout;
#else
    yyin = (FILE *) 0;
    yyout = (FILE *) 0;
#endif

    /* For future reference: Set errno on error, since we are called by
     * yylex_init()
     */
    return 0;
}

/* yylex_destroy is for both reentrant and non-reentrant scanners. */
int yylex_destroy(void)
{

    /* Pop the buffer stack, destroying each element. */
    while (YY_CURRENT_BUFFER) {
        yy_delete_buffer(YY_CURRENT_BUFFER);
        YY_CURRENT_BUFFER_LVALUE = NULL;
        yypop_buffer_state();
    }

    /* Destroy the stack itself. */
    yyfree((yy_buffer_stack));
    (yy_buffer_stack) = NULL;

    /* Reset the globals. This is important in a non-reentrant scanner so the next time
     * yylex() is called, initialization will occur. */
    yy_init_globals();

    return 0;
}

/*
 * Internal utility routines.
 */

#ifndef yytext_ptr
static void yy_flex_strncpy(char* s1, yyconst char * s2, int n)
{
    register int i;
    for (i = 0; i < n; ++i)
        s1[i] = s2[i];
}
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen(yyconst char * s)
{
    register int n;
    for (n = 0; s[n]; ++n)
        ;

    return n;
}
#endif

void *yyalloc(yy_size_t  size)
{
    return (void *) malloc(size);
}

void *yyrealloc(void * ptr, yy_size_t  size)
{
    /* The cast to (char *) in the following accommodates both
     * implementations that use char* generic pointers, and those
     * that use void* generic pointers.  It works with the latter
     * because both ANSI C and C++ allow castless assignment from
     * any pointer type to void*, and deal with argument conversions
     * as though doing an assignment.
     */
    return (void *) realloc((char *) ptr, size);
}

void yyfree(void * ptr)
{
    free((char *) ptr);   /* see yyrealloc() for (char *) cast */
}

#define YYTABLES_NAME "yytables"

#line 310 "sqlscanner.l"



void tokenize(const char *data)
{
    yy_switch_to_buffer(yy_scan_string(data));
    ctoken = "";
    current = 0;
}


